import {
  useFetchFrameApp
} from "./chunk-BXR2F7XL.js";
import {
  useResolveClient
} from "./chunk-DIJQCH5I.js";
import {
  useDebugLog
} from "./chunk-TLOKKM4F.js";
import {
  useFreshRef
} from "./chunk-KTZM67PO.js";
import {
  assertNever
} from "./chunk-YAAIMQMX.js";

// src/use-frame-app.ts
import { useMemo } from "react";
var defaultFrameRequestCache = /* @__PURE__ */ new Set();
var defaultOnAddFrameRequested = function defaultOnAddFrameRequested2() {
  console.error(
    "@frames.js/render/use-frame-app",
    "onAddFrameRequested not implemented"
  );
  return Promise.reject(new Error("onAddFrameRequested not implemented"));
};
var defaultOnSendTransactionRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSendTransactionRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultOnSignMessageRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSignMessageRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultOnSignTypedDataRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSignTypedDataRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultLocation = {
  type: "launcher"
};
function useFrameApp({
  provider,
  client,
  location = defaultLocation,
  farcasterSigner,
  source,
  fetchFn,
  proxyUrl,
  onClose,
  onOpenUrl,
  onPrimaryButtonSet,
  onReady,
  debug = false,
  addFrameRequestsCache = defaultFrameRequestCache,
  onAddFrameRequested = defaultOnAddFrameRequested,
  onSendTransactionRequest = defaultOnSendTransactionRequest,
  onSignMessageRequest = defaultOnSignMessageRequest,
  onSignTypedDataRequest = defaultOnSignTypedDataRequest
}) {
  const providerRef = useFreshRef(provider);
  const debugRef = useFreshRef(debug);
  const locationRef = useFreshRef(location);
  const readyRef = useFreshRef(onReady);
  const closeRef = useFreshRef(onClose);
  const onOpenUrlRef = useFreshRef(onOpenUrl);
  const onPrimaryButtonSetRef = useFreshRef(onPrimaryButtonSet);
  const farcasterSignerRef = useFreshRef(farcasterSigner);
  const onAddFrameRequestedRef = useFreshRef(onAddFrameRequested);
  const addFrameRequestsCacheRef = useFreshRef(addFrameRequestsCache);
  const clientResolutionState = useResolveClient({ client });
  const frameResolutionState = useFetchFrameApp({
    source,
    fetchFn,
    proxyUrl
  });
  const logDebug = useDebugLog(
    "@frames.js/render/use-frame-app",
    debugRef.current
  );
  providerRef.current.setEventHandlers({
    onSendTransactionRequest,
    onSignMessageRequest,
    onSignTypedDataRequest
  });
  return useMemo(() => {
    if (clientResolutionState.status === "pending") {
      return {
        status: "pending"
      };
    }
    if (clientResolutionState.status === "error") {
      return {
        status: "error",
        error: clientResolutionState.error
      };
    }
    if (farcasterSignerRef.current.signer?.status !== "approved" && farcasterSignerRef.current.signer?.status !== "impersonating") {
      return {
        status: "error",
        error: new Error(
          "Farcaster signer must be either approved or impersonating"
        )
      };
    }
    const signer = farcasterSignerRef.current.signer;
    const resolvedClient = clientResolutionState.client;
    switch (frameResolutionState.status) {
      case "success": {
        const frame = frameResolutionState.frame;
        return {
          getEmitter: (endpoint) => ({
            emit: endpoint.emit,
            emitEthProvider: endpoint.emitEthProvider
          }),
          sdk: (endpoint) => ({
            async addFrame() {
              logDebug("sdk.addFrame() called");
              if (frame.status !== "success" || frame.manifest?.status !== "success") {
                logDebug("Invalid frame domain manifest");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "invalid_domain_manifest"
                });
                return {
                  added: false,
                  reason: "invalid_domain_manifest"
                };
              }
              if (addFrameRequestsCacheRef.current.has(
                frame.frame.button.action.url
              )) {
                logDebug("Frame already requested to be added.");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "rejected_by_user"
                });
                return {
                  added: false,
                  reason: "rejected_by_user"
                };
              }
              const added = await onAddFrameRequestedRef.current(frame);
              logDebug("onAddFrameRequested() called", added);
              addFrameRequestsCacheRef.current.add(
                frame.frame.button.action.url
              );
              if (!added) {
                logDebug("Frame add request rejected by user");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "rejected_by_user"
                });
                return {
                  added: false,
                  reason: "rejected_by_user"
                };
              }
              endpoint.emit({
                event: "frame_added",
                notificationDetails: added.notificationDetails
              });
              return added;
            },
            close() {
              logDebug("sdk.close() called");
              closeRef.current?.();
            },
            context: {
              client: resolvedClient,
              location: locationRef.current,
              user: { fid: signer.fid }
            },
            async ethProviderRequest(parameters) {
              return providerRef.current.request(parameters);
            },
            openUrl(url) {
              logDebug("sdk.openUrl() called", url);
              onOpenUrlRef.current?.(url);
            },
            ready(options) {
              logDebug("sdk.ready() called");
              readyRef.current?.(options);
            },
            setPrimaryButton(options) {
              logDebug("sdk.setPrimaryButton() called", options);
              onPrimaryButtonSetRef.current?.(options, () => {
                logDebug('"primaryButtonClicked" called');
                endpoint.emit({
                  event: "primary_button_clicked"
                });
              });
            },
            signIn() {
              throw new Error("not implemented");
            }
          }),
          status: "success",
          frame: frameResolutionState.frame,
          client: clientResolutionState.client
        };
      }
      case "error": {
        return {
          status: "error",
          error: frameResolutionState.error
        };
      }
      case "pending": {
        return {
          status: "pending"
        };
      }
      default:
        assertNever(frameResolutionState);
    }
  }, [
    clientResolutionState,
    farcasterSignerRef,
    frameResolutionState,
    locationRef,
    logDebug,
    addFrameRequestsCacheRef,
    onAddFrameRequestedRef,
    closeRef,
    providerRef,
    onOpenUrlRef,
    readyRef,
    onPrimaryButtonSetRef
  ]);
}

export {
  useFrameApp
};
