import { HostEndpoint } from '@farcaster/frame-host';
import { FrameContext, SetPrimaryButton, AddFrameResult } from '@farcaster/frame-sdk';
import { ParseFramesV2ResultWithFrameworkDetails } from 'frames.js/frame-parsers';
import { Provider } from 'ox/Provider';
import { ExtractRequest, Default } from 'ox/RpcSchema';

type FrameClientConfig = FrameContext["client"];
type SendTransactionRpcRequest = ExtractRequest<Default, "eth_sendTransaction">;
type SignMessageRpcRequest = ExtractRequest<Default, "personal_sign">;
type SignTypedDataRpcRequest = ExtractRequest<Default, "eth_signTypedData_v4">;
type EthProvider = Provider<undefined, Default> & {
    setEventHandlers: (handlers: SharedEthProviderEventHandlers) => void;
};
/**
 * Function called when the frame app requests sending transaction.
 *
 * If false is returned then the request is cancelled and user rejected error is thrown
 */
type OnSendTransactionRequestFunction = (request: SendTransactionRpcRequest) => Promise<boolean>;
/**
 * Function called when the frame app requests signing message.
 *
 * If false is returned signing is cancelled and user rejected error is thrown
 */
type OnSignMessageRequestFunction = (request: SignMessageRpcRequest) => Promise<boolean>;
/**
 * Function called when the frame app requests signing typed data.
 *
 * If false is returned then the request is cancelled and user rejected error is thrown
 */
type OnSignTypedDataRequestFunction = (request: SignTypedDataRpcRequest) => Promise<boolean>;
type SharedEthProviderEventHandlers = {
    onSendTransactionRequest: OnSendTransactionRequestFunction;
    onSignMessageRequest: OnSignMessageRequestFunction;
    onSignTypedDataRequest: OnSignTypedDataRequestFunction;
};
type FramePrimaryButton = Parameters<SetPrimaryButton>[0];
type OnPrimaryButtonSetFunction = (options: FramePrimaryButton, pressedCallback: () => void) => void;
type OnAddFrameRequestedFunction = (frame: ParseFramesV2ResultWithFrameworkDetails) => Promise<false | Extract<AddFrameResult, {
    added: true;
}>>;
/**
 * Function called when the frame app is being loaded and we need to resolve the client that renders the frame app
 */
type ResolveClientFunction = (options: {
    signal: AbortSignal;
}) => Promise<FrameClientConfig>;
type HostEndpointEmitter = Pick<HostEndpoint, "emit" | "emitEthProvider">;

export type { EthProvider, FrameClientConfig, FramePrimaryButton, HostEndpointEmitter, OnAddFrameRequestedFunction, OnPrimaryButtonSetFunction, OnSendTransactionRequestFunction, OnSignMessageRequestFunction, OnSignTypedDataRequestFunction, ResolveClientFunction, SendTransactionRpcRequest, SharedEthProviderEventHandlers, SignMessageRpcRequest, SignTypedDataRpcRequest };
